* Digital library for 0.18u/1.8V predictive technology model

* Inverter 
.subckt inv in out vdd vss
xmp (out in vdd vdd) submodp w=1.0u l=0.18u
xmn (out in vss vss) submodn w=0.5u l=0.18u
.ends

* Current-starved 3-stage ring-oscillator (VCO)
.subckt ringvco ctl out vdd vss
* Ring
xmp1 (b a vdd1 vdd) submodp w=6u l=0.3u
xmn1 (b a vss1 vss) submodn w=2u l=0.3u
xmp2 (c b vdd2 vdd) submodp w=6u l=0.3u
xmn2 (c b vss2 vss) submodn w=2u l=0.3u
xmp3 (a c vdd3 vdd) submodp w=6u l=0.3u
xmn3 (a c vss3 vss) submodn w=2u l=0.3u
* Pmos current mirrors
xmpref (ctlp ctlp vdd vdd) submodp w=2.4u l=0.2u
xmpm1  (vdd1 ctlp vdd vdd) submodp w=2.4u l=0.2u
xmpm2  (vdd2 ctlp vdd vdd) submodp w=2.4u l=0.2u
xmpm3  (vdd3 ctlp vdd vdd) submodp w=2.4u l=0.2u
* Nmos current mirrors  
xmnref (ctlp ctl  vss vss) submodn w=0.8u l=0.2u
xmnm1  (vss1 ctl  vss vss) submodn w=0.8u l=0.2u
xmnm2  (vss2 ctl  vss vss) submodn w=0.8u l=0.2u
xmnm3  (vss3 ctl  vss vss) submodn w=0.8u l=0.2u
* Output buffer (2 inverters)
xmpb1 (c1  c  vdd vdd) submodp w=1.08u l=0.18u
xmnb1 (c1  c  vss vss) submodn w=0.36u l=0.18u
xmpb2 (out c1 vdd vdd) submodp w=1.08u l=0.18u
xmnb2 (out c1 vss vss) submodn w=0.36u l=0.18u
* Startup
* xmstart (a lev vdd vdd) submodp w=2.4u l=0.2u
* rstart (lev vdd) r=0.2k
* cstart (lev vss) c=0.5p
.ends

* NAND gate
.subckt nand a b out vdd vss
* PMOS                                     
xmpa (out a vdd vdd) submodp w=1.0u l=0.18u
xmpb (out b vdd vdd) submodp w=1.0u l=0.18u                     
* NMOS                                                          
xmna (out a t   vss) submodn w=0.5u l=0.18u                     
xmnb (t   b vss vss) submodn w=0.5u l=0.18u                     
.ends                
                     
* 3-input NAND gate
.subckt nand3 a b c out vdd vss
* PMOS
xmpa (out a vdd vdd) submodp w=1u l=0.18u
xmpb (out b vdd vdd) submodp w=1u l=0.18u
xmpc (out c vdd vdd) submodp w=1u l=0.18u
* NMOS
xmna (out a t1  vss) submodn w=0.5u l=0.18u
xmnb (t1  b t2  vss) submodn w=0.5u l=0.18u
xmnc (t2  c vss vss) submodn w=0.5u l=0.18u
.ends

* NOR gate
.subckt nor a b out vdd vss
* PMOS
xmpa (t   a vdd vdd) submodp w=1u l=0.18u
xmpb (out b t   vdd) submodp w=1u l=0.18u
* NMOS
xmna (out a vss vss) submodn w=0.5u l=0.18u
xmnb (out b vss vss) submodn w=0.5u l=0.18u
.ends

* XOR gate
.subckt xor a b out vdd vss
* Invert a
xmiap  (na a vdd vdd) submodp w=1u l=0.18u
xmian  (na a vss vss) submodn w=0.5u l=0.18u
* Invert b
xmibp  (nb b vdd vdd) submodp w=1u l=0.18u
xmibn  (nb b vss vss) submodn w=0.5u l=0.18u
* PMOS
xmpna (t1  na vdd vdd) submodp w=1u l=0.18u 
xmpb  (out b  t1  vdd) submodp w=1u l=0.18u 
xmpa  (t2  a  vdd vdd) submodp w=1u l=0.18u 
xmpnb (out nb t2  vdd) submodp w=1u l=0.18u 
* NMOS
xmnna (out na t   vss) submodn w=0.5u l=0.18u
xmnb  (out b  t   vss) submodn w=0.5u l=0.18u
xmna  (t   a  vss vss) submodn w=0.5u l=0.18u
xmnnb (t   nb vss vss) submodn w=0.5u l=0.18u
.ends

* Rising clock edge triggered d flip flop with async. reset (active low)
.subckt dff clk d clrn q qn vdd vss
xnand1 (b  sn       a  vdd vss) nand
xnand2 (a  clk clrn sn vdd vss) nand3
xnand3 (sn clk b    rn vdd vss) nand3
xnand4 (rn d   clrn b  vdd vss) nand3
xnand5 (sn qn       q  vdd vss) nand
xnang6 (q  rn  clrn qn vdd vss) nand3
.ends

* Rising clock edge triggered t flip flop with async. reset (active low)
.subckt tff clk t cln q qn vdd vss
xdff (clk d cln q qn vdd vss) dff
xxor (q t d vdd vss) xor
.ends

* 3-bit synchronous counter
.subckt cnt3bit clk q1 q2 q3 vdd vss
* t flip flops
xtff1 (clk vdd vdd q1 qn1 vdd vss) tff
xtff2 (clk q1  vdd q2 qn2 vdd vss) tff
xtff3 (clk t3  vdd q3 qn3 vdd vss) tff
* t signal for the 3rd flip flop
xnand (q1 q2 t3x vdd vss) nand
xinv  (t3x t3 vdd vss) inv
.ends

* Phase frequency detector
.subckt pfd a b up dn vdd vss
xdff1 (a vdd rstn qup qupn vdd vss) dff
xdff2 (b vdd rstn qdn qdnn vdd vss) dff
xnand (qup qdn rstn vdd vss) nand
xnorup (qupn qdn up vdd vss) nor
xnordn (qdnn qup dn vdd vss) nor
.ends

* Simple charge pump 
.subckt chgpump up dn out vdd vss
xmup (out upn vdd vdd) submodp w=0.90u l=0.18u
xmun (out dn  vss vss) submodn w=0.36u l=0.18u
xiup (up upn vdd vss) inv
rd1 (vdd  out) r=100k
rd2 (out vss)  r=100k
.ends

* Filter (high cutoff)
.subckt filterfast in out vss
r1 (in out)  r=16k
c1 (out a)   c=0.1p
r2 (a vss)   r=5k
c2 (out vss) c=1p
.ends

.subckt filter in out vss
r1 (in out)  r=655.2k
c1 (out a)   c=1p
r2 (a vss)   r=197.2k
c2 (out vss) c=20p
.ends
